@@ -35,6 +35,8 @@
 #include <clutter/clutter.h>
 #endif
 
+#include <libgames-support/games-debug.h>
+#include <libgames-support/games-glib-compat.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-string-utils.h>
 
@@ -43,15 +45,9 @@
 
 #include "game.h"
 
-#include "board.h"
-
 #define DELAYED_CALLBACK_DELAY (50)
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
 #define I_(string) g_intern_static_string (string)
-#else
-#define I_(string) string
-#endif
 
 struct _AisleriotGame
 {
@@ -238,7 +234,7 @@ clear_slots (AisleriotGame *game,
 
   n_slots = game->slots->len;
   for (i = 0; i < n_slots; ++i) {
-    Slot *slot = game->slots->pdata[i];
+    ArSlot *slot = game->slots->pdata[i];
 
 #ifdef HAVE_CLUTTER
     if (slot->slot_renderer) {
@@ -252,11 +248,7 @@ clear_slots (AisleriotGame *game,
 #endif /* HAVE_CLUTTER */
     g_byte_array_free (slot->cards, TRUE);
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
-    g_slice_free (Slot, slot);
-#else
-    g_free (slot);
-#endif
+    g_slice_free (ArSlot, slot);
   }
 
   g_ptr_array_set_size (game->slots, 0);
@@ -266,7 +258,7 @@ clear_slots (AisleriotGame *game,
   }
 }
 
-static Slot *
+static ArSlot *
 get_slot (AisleriotGame *game,
           gint slotid)
 {
@@ -274,7 +266,7 @@ get_slot (AisleriotGame *game,
 
   n_slots = game->slots->len;
   for (i = 0; i < n_slots; ++i) {
-    Slot *hslot = game->slots->pdata[i];
+    ArSlot *hslot = game->slots->pdata[i];
 
     if (hslot->id == slotid)
       return hslot;
@@ -346,7 +338,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
   n_slots = slots->len;
   if (n_slots > 0) {
     for (i = 0; i < n_slots; ++i) {
-      Slot *slot = slots->pdata[i];
+      ArSlot *slot = slots->pdata[i];
       GByteArray *cards = slot->cards;
       guint n_cards;
 
@@ -528,7 +520,7 @@ c2scm_deck (guint8 *cards,
 }
 
 static void
-cscmi_slot_set_cards (Slot *slot,
+cscmi_slot_set_cards (ArSlot *slot,
                       SCM cards)
 {
   AisleriotGame *game = app_game;
@@ -568,10 +560,12 @@ static SCM
 cscmi_add_slot (SCM slot_data)
 {
   AisleriotGame *game = app_game;
-  Slot *slot;
+  ArSlot *slot;
   gboolean expanded_down = FALSE;
   gboolean expanded_right = FALSE;
   int expansion_depth = 0;
+  ArSlotType type = AR_SLOT_UNKNOWN;
+  SCM slot_placement, slot_type;
 
   if (game->state > GAME_BEGIN) {
     return scm_throw (scm_from_locale_symbol ("game-started"),
@@ -580,35 +574,61 @@ cscmi_add_slot (SCM slot_data)
   }
 
 #ifdef HAVE_GUILE_1_8
-#define CHECK_EXPANSION(string,object) (scm_is_true (scm_equal_p (scm_from_locale_symbol (string), object)))
+#define EQUALS_SYMBOL(string,object) (scm_is_true (scm_equal_p (scm_from_locale_symbol (string), object)))
 #else
-#define CHECK_EXPANSION(string,object) (!strcmp (string, SCM_CHARS (object)))
+#define EQUALS_SYMBOL(string,object) (!strcmp (string, SCM_CHARS (object)))
 #endif
 
-  if (CHECK_EXPANSION ("expanded", SCM_CAR (SCM_CADDR (slot_data)))) {
+  slot_placement = SCM_CADDR (slot_data);
+  if (EQUALS_SYMBOL ("expanded", SCM_CAR (slot_placement))) {
     expanded_down = TRUE;
-  } else if (CHECK_EXPANSION ("expanded-right", SCM_CAR (SCM_CADDR (slot_data)))) {
+  } else if (EQUALS_SYMBOL ("expanded-right", SCM_CAR (slot_placement))) {
     expanded_right = TRUE;
-  } else if (CHECK_EXPANSION ("partially-expanded", SCM_CAR (SCM_CADDR (slot_data)))) {
+  } else if (EQUALS_SYMBOL ("partially-expanded", SCM_CAR (slot_placement))) {
     expanded_down = TRUE;
-    expansion_depth = scm_to_int (SCM_CADDR (SCM_CADDR (slot_data)));
-  } else if (CHECK_EXPANSION ("partially-expanded-right", SCM_CAR (SCM_CADDR (slot_data)))) {
+    expansion_depth = scm_to_int (SCM_CADDR (slot_placement));
+  } else if (EQUALS_SYMBOL ("partially-expanded-right", SCM_CAR (slot_placement))) {
     expanded_right = TRUE;
-    expansion_depth = scm_to_int (SCM_CADDR (SCM_CADDR (slot_data)));
+    expansion_depth = scm_to_int (SCM_CADDR (slot_placement));
   }
 
 #undef CHECK_EXPANSION
 
+  /* 3rd argument is the slot type (optionally) */
+  slot_type = SCM_CDDDR (slot_data);
+  if (slot_type != SCM_EOL) {
+    if (EQUALS_SYMBOL ("foundation", SCM_CAR (slot_type))) {
+      type = AR_SLOT_FOUNDATION;
+    } else if (EQUALS_SYMBOL ("reserve", SCM_CAR (slot_type))) {
+      type = AR_SLOT_RESERVE;
+    } else if (EQUALS_SYMBOL ("stock", SCM_CAR (slot_type))) {
+      type = AR_SLOT_STOCK;
+    } else if (EQUALS_SYMBOL ("tableau", SCM_CAR (slot_type))) {
+      type = AR_SLOT_TABLEAU;
+    } else if (EQUALS_SYMBOL ("waste", SCM_CAR (slot_type))) {
+      type = AR_SLOT_WASTE;
+    }
+  }
+
+#undef EQUALS_SYMBOL
+
+#ifdef GNOME_ENABLE_DEBUG
+  _GAMES_DEBUG_IF (GAMES_DEBUG_SCHEME) {
+    static const char *types[] = { "unknown", "foundation", "reserve", "stock", "tableau", "waste" };
+
+    _games_debug_print (GAMES_DEBUG_SCHEME,
+                        "Adding new slot %d type %s\n",
+                        scm_to_int (SCM_CAR (slot_data)), types[type]);
+  }
+#endif /* GNOME_ENABLE_DEBUG */
+
   /* create and initialize slot */
-#if GLIB_CHECK_VERSION (2, 10, 0)
-  slot = g_slice_new0 (Slot);
-#else
-  slot = g_new0 (Slot, 1);
-#endif
+  slot = g_slice_new0 (ArSlot);
 
   g_ptr_array_add (game->slots, slot);
 
   slot->id = scm_to_int (SCM_CAR (slot_data));
+  slot->type = type;
 
   slot->cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
   slot->exposed = 0;
@@ -756,7 +776,7 @@ scm_set_slot_x_expansion (SCM scm_slot_id,
                           SCM new_exp_val)
 {
   AisleriotGame *game = app_game;
-  Slot *slot;
+  ArSlot *slot;
 
   slot = get_slot (game, scm_num2int (scm_slot_id, SCM_ARG1, NULL));
 
@@ -780,7 +800,7 @@ scm_set_slot_y_expansion (SCM scm_slot_id,
                           SCM new_exp_val)
 {
   AisleriotGame *game = app_game;
-  Slot *slot;
+  ArSlot *slot;
 
   slot = get_slot (game, scm_num2int (scm_slot_id, SCM_ARG1, NULL));
 
@@ -803,7 +823,7 @@ static SCM
 scm_get_slot (SCM scm_slot_id)
 {
   AisleriotGame *game = app_game;
-  Slot *slot;
+  ArSlot *slot;
 
   slot = get_slot (game, scm_num2int (scm_slot_id, SCM_ARG1, NULL));
 
@@ -820,7 +840,7 @@ scm_set_cards (SCM scm_slot_id,
                SCM new_cards)
 {
   AisleriotGame *game = app_game;
-  Slot *slot;
+  ArSlot *slot;
 
   slot = get_slot (game, scm_num2int (scm_slot_id, SCM_ARG1, NULL));
 
@@ -1007,11 +1027,13 @@ scm_delayed_call (SCM callback)
 static void
 cscm_init (void)
 {
+#ifdef GUILE_DEBUG
   /* Enable useful debugging options. */
   SCM_DEVAL_P = 1;
   SCM_BACKTRACE_P = 1;
   SCM_RECORD_POSITIONS_P = 1;
   SCM_RESET_DEBUG_MODE;
+#endif
 
   /* Let the scheme side of things know about our C functions. */
   scm_c_define_gsubr ("set-feature-word!", 1, 0, 0, scm_set_feature_word);
@@ -1308,42 +1330,48 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
      PROP_CAN_UNDO,
      g_param_spec_boolean ("can-undo", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_CAN_REDO,
      g_param_spec_boolean ("can-redo", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_CAN_DEAL,
      g_param_spec_boolean ("can-deal", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_GAME_FILE,
      g_param_spec_string ("game-file", NULL, NULL,
                           NULL,
-                          G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_SCORE,
-     g_param_spec_uint ("score", "", "",
+     g_param_spec_uint ("score", NULL, NULL,
                         0, G_MAXUINT, 0,
-                        G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_STATE,
-     g_param_spec_uint ("state", "", "",
+     g_param_spec_uint ("state", NULL, NULL,
                         0, LAST_GAME_STATE, 0, 
-                        G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS));
 
   /* Initialise our scheme interfaces */
   cscm_init ();
@@ -1351,6 +1379,112 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
 
 /* public API */
 
+/**
+ * ar_slot_get_slot_type:
+ * @slot: a #ArSlot
+ *
+ * Returns: the slot type of @slot
+ */
+ArSlotType
+ar_slot_get_slot_type (ArSlot *slot)
+{
+  g_return_val_if_fail (slot != NULL, AR_SLOT_UNKNOWN);
+
+  return slot->type;
+}
+
+/**
+ * ar_slot_get_type_string:
+ * @slot: a #ArSlot
+ *
+ * Returns: a string describing the slot type
+ */
+const char *
+ar_slot_get_type_string (ArSlot *slot)
+{
+  const char *text = NULL;
+
+  g_return_val_if_fail (slot != NULL, NULL);
+
+  switch (slot->type) {
+    case AR_SLOT_UNKNOWN:
+      text = NULL;
+      break;
+    case AR_SLOT_FOUNDATION:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "foundation");
+      break;
+    case AR_SLOT_RESERVE:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "reserve");
+      break;
+    case AR_SLOT_STOCK:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "stock");
+      break;
+    case AR_SLOT_TABLEAU:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "tableau");
+      break;
+    case AR_SLOT_WASTE:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "waste");
+      break;
+  }
+
+  return text;
+}
+
+/**
+ * ar_slot_get_hint_string:
+ * @slot: a #ArSlot
+ *
+ * Returns: a string describing the slot type
+ */
+char *
+ar_slot_get_hint_string (ArSlot *slot,
+                         int cardid)
+{
+  const char *card_name;
+
+  g_return_val_if_fail (slot != NULL, NULL);
+
+  if (slot->type == AR_SLOT_UNKNOWN)
+    return NULL;
+
+  if (cardid < 0)
+    return g_strdup (ar_slot_get_type_string (slot));
+
+  card_name = games_card_get_locale_name (CARD (slot->cards->data[cardid]));
+
+  switch (slot->type) {
+    case AR_SLOT_FOUNDATION:
+      /* Translators: %s is the name of the card; "foundation" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on foundation"), card_name);
+
+    case AR_SLOT_RESERVE:
+      /* Translators: %s is the name of the card; "reserve" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on reserve"), card_name);
+
+    case AR_SLOT_STOCK:
+      /* Translators: %s is the name of the card; "stock" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on stock"), card_name);
+
+    case AR_SLOT_TABLEAU:
+      /* Translators: %s is the name of the card; "tableau" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on tableau"), card_name);
+
+    case AR_SLOT_WASTE:
+      /* Translators: %s is the name of the card; "waste" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on waste"), card_name);
+
+    default:
+      g_assert_not_reached ();
+  }
+
+  return NULL;
+}
+
 /**
  * aisleriot_game_error_quark:
  *
@@ -1404,7 +1538,7 @@ aisleriot_game_get_slots (AisleriotGame *game)
  */
 void
 aisleriot_game_slot_add_cards (AisleriotGame *game,
-                               Slot *slot,
+                               ArSlot *slot,
                                guint8 *cards,
                                guint n_cards)
 {
@@ -2051,11 +2185,7 @@ aisleriot_game_option_free (AisleriotGameOption *option)
   g_return_if_fail (option != NULL);
 
   g_free (option->display_name);
-#if GLIB_CHECK_VERSION (2, 10, 0)
   g_slice_free (AisleriotGameOption, option);
-#else
-  g_free (option);
-#endif
 }
 
 /**
@@ -2112,12 +2242,7 @@ aisleriot_game_get_options (AisleriotGame *game)
 
       entrystate = SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1)));
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
       option = g_slice_new (AisleriotGameOption);
-#else
-      option = g_new (AisleriotGameOption, 1);
-#endif
-
       option->display_name = g_strdup (entrynamestr);
       option->type = type;
       option->value = bit;
@@ -2361,7 +2486,7 @@ aisleriot_game_deal_cards (AisleriotGame *game)
 #ifdef HAVE_CLUTTER
 
 void
-aisleriot_game_get_card_offset (Slot *slot,
+aisleriot_game_get_card_offset (ArSlot *slot,
                                 guint card_num,
                                 gboolean old_cards,
                                 gint *xoff, gint *yoff)
@@ -2387,7 +2512,7 @@ aisleriot_game_get_card_offset (Slot *slot,
 }
 
 void
-aisleriot_game_reset_old_cards (Slot *slot)
+aisleriot_game_reset_old_cards (ArSlot *slot)
 {
   g_byte_array_set_size (slot->old_cards, 0);
   g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
