@@ -1,19 +1,18 @@
 /*
- *  Copyright © 2007 Christian Persch
+ * Copyright © 2007 Christian Persch
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope game it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef AISLERIOT_GAME_H
@@ -31,8 +30,18 @@ G_BEGIN_DECLS
 
 /* A slot */
 
+typedef enum {
+  AR_SLOT_UNKNOWN,
+  AR_SLOT_FOUNDATION,
+  AR_SLOT_RESERVE,
+  AR_SLOT_STOCK,
+  AR_SLOT_TABLEAU,
+  AR_SLOT_WASTE
+} ArSlotType;
+
 typedef struct {
   int id;
+  ArSlotType type;
 
   GByteArray *cards;
 #ifdef HAVE_CLUTTER
@@ -74,10 +83,17 @@ typedef struct {
   guint dx_set : 1;
   guint dy_set : 1;
   guint needs_update : 1;
-} Slot;
+} ArSlot;
 
 #define SLOT_CARDS_N_PREALLOC (32)
 
+ArSlotType ar_slot_get_slot_type (ArSlot *slot);
+
+const char *ar_slot_get_type_string (ArSlot *slot);
+
+char *ar_slot_get_hint_string (ArSlot *slot,
+                               int cardid);
+
 /* The game */
 
 #define AISLERIOT_TYPE_GAME         (aisleriot_game_get_type ())
@@ -142,7 +158,7 @@ AisleriotGame *aisleriot_game_new (void);
 GPtrArray *aisleriot_game_get_slots (AisleriotGame * game);
 
 void aisleriot_game_slot_add_cards (AisleriotGame * game,
-                                    Slot * slot,
+                                    ArSlot * slot,
                                     guint8 * cards, guint n_cards);
 
 guint aisleriot_game_get_state (AisleriotGame * game);
@@ -219,12 +235,12 @@ void aisleriot_game_generate_exception (AisleriotGame * game);
 
 void aisleriot_game_deal_cards (AisleriotGame *game);
 
-void aisleriot_game_get_card_offset (Slot *slot,
+void aisleriot_game_get_card_offset (ArSlot *slot,
                                      guint card_num,
                                      gboolean old_cards,
                                      gint *xoff, gint *yoff);
 
-void aisleriot_game_reset_old_cards (Slot *slot);
+void aisleriot_game_reset_old_cards (ArSlot *slot);
 
 G_END_DECLS
 
