@@ -26,6 +26,8 @@
 #include "slot-renderer.h"
 #include "card.h"
 
+#include <libgames-support/games-glib-compat.h>
+
 static void aisleriot_slot_renderer_dispose (GObject *object);
 static void aisleriot_slot_renderer_finalize (GObject *object);
 
@@ -111,7 +113,8 @@ sync_style_selection_color (ArStyle *style,
 
   priv->highlight_color = color;
 
-  /* FIXMEchpe: queue a redraw if necessary! */
+  if (priv->show_highlight)
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (srend));
 }
 
 static void
@@ -153,7 +156,7 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
 
   pspec = g_param_spec_int ("highlight", NULL, NULL,
                             -1, G_MAXINT, G_MAXINT,
-                               G_PARAM_READWRITE |
+                            G_PARAM_READWRITE |
                             G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (gobject_class, PROP_HIGHLIGHT, pspec);
 
