@@ -69,6 +69,10 @@
 
 #include "window.h"
 
+#ifdef ENABLE_DEBUG_UI
+#include "prop-editor.h"
+#endif
+
 #define AISLERIOT_WINDOW_GET_PRIVATE(window)(G_TYPE_INSTANCE_GET_PRIVATE ((window), AISLERIOT_TYPE_WINDOW, AisleriotWindowPrivate))
 
 #define MIN_WIDTH 600
@@ -105,13 +109,6 @@
 #define LEAVE_FULLSCREEN_BUTTON
 #endif
 
-/* define this to enable a debug menu */
-/* #undef ENABLE_DEBUG_UI */
-
-#ifdef ENABLE_DEBUG_UI
-#include "prop-editor.h"
-#endif
-
 enum
 {
   ACTION_UNDO_MOVE,
@@ -2183,9 +2180,17 @@ aisleriot_window_state_event (GtkWidget *widget,
 #ifdef HAVE_HILDON
 /* We never show tooltips, no need to put them into the binary */
 #define ACTION_TOOLTIP(string)  (NULL)
-#define ACTION_ACCEL(string1,string2) (string2)
 #else
 #define ACTION_TOOLTIP(string)  (string)
+#endif
+
+/* The maemo5 device (N900) doesn't have these extra hardware keys,
+ * so we only re-define the accels for maemo[34]. The N900 does have
+ * a slide-out keyboard, so using the regular accels should work fine.
+ */
+#if defined(HAVE_HILDON) && (defined(HAVE_MAEMO_3) || defined(HAVE_MAEMO_4))
+#define ACTION_ACCEL(string1,string2) (string2)
+#else
 #define ACTION_ACCEL(string1,string2) (string1)
 #endif
 
@@ -2213,7 +2218,7 @@ aisleriot_window_init (AisleriotWindow *window)
       ACTION_TOOLTIP (N_("Play a different game")),
       G_CALLBACK (select_game_cb) },
     { "RecentMenu", NULL, N_("_Recently Played") },
-    { "Statistics", GTK_STOCK_ADD, N_("S_tatistics"), NULL,
+    { "Statistics", NULL, N_("S_tatistics"), NULL,
       ACTION_TOOLTIP (N_("Show gameplay statistics")),
       G_CALLBACK (statistics_cb) },
     { "CloseWindow", GTK_STOCK_CLOSE, NULL, NULL,
