@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import struct
 
 class GGZChessFeedback:
@@ -85,7 +86,9 @@ class Chess:
             return (None, 0)
         
         string = buffer[4:length + 4]
-        if string[-1] == '\x00':
+        
+        # Strip C null characters
+        while string.endswith('\x00'):
             string = string[:-1]
         
         return (string, length + 4)
