@@ -20,11 +20,16 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
 #include <string.h>
-#include <games-stock.h>
-#include <games-scores.h>
-#include <games-scores-dialog.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
+#include <libgames-support/games-help.h>
+#include <libgames-support/games-stock.h>
+#include <libgames-support/games-scores.h>
+#include <libgames-support/games-scores-dialog.h>
 
 #include "gbdefs.h"
 #include "menu.h"
@@ -233,7 +238,7 @@ quit_cb (GtkAction * action, gpointer data)
 static void
 help_cb (GtkAction * action, gpointer data)
 {
-  gnome_help_display ("gnobots2.xml", NULL, NULL);
+  games_help_display (app, "gnobots2", NULL);
 }
 
 
@@ -260,7 +265,7 @@ about_cb (GtkAction * action, gpointer data)
   gtk_show_about_dialog (GTK_WINDOW (app),
 			 "name", _("Robots"),
 			 "version", VERSION,
-			 "copyright", "Copyright \xc2\xa9 1998-2007 Mark Rae",
+			 "copyright", "Copyright \xc2\xa9 1998-2008 Mark Rae",
 			 "license", license,
 		         "website-label", _("GNOME Games web site"),
 			 "comments", _("Based on classic BSD Robots.\n\nRobots is a part of GNOME Games."),
@@ -368,21 +373,6 @@ create_game_menus (GtkUIManager * ui_manager)
 
 /**********************************************************************/
 
-/**
- * update_score_state
- *
- * Description:
- * Changes menu item enabled/disabled state depending on high score availability
- **/
-void
-update_score_state (void)
-{
-  GList *top;
-
-  top = games_scores_get (highscores);
-  gtk_action_set_sensitive (scores_action, top != NULL);
-}
-
 void
 set_move_menu_sensitivity (gboolean state)
 {
@@ -413,5 +403,5 @@ connect_toolbar_toggle (GtkWidget * toolbar)
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toolbar_toggle_action),
 				properties_show_toolbar ());
 
-  toggle_toolbar_cb (GTK_TOGGLE_ACTION (toolbar_toggle_action), toolbar);
+  toggle_toolbar_cb (GTK_ACTION (toolbar_toggle_action), toolbar);
 }
